#include <stdio.h>
#include <stdlib.h>

int get_num_threads();
int get_thread_id();

int main() {

  int nthreads = -1;

  nthreads = get_num_threads();    
  printf("\nSerial: %d\n\n",nthreads);

  // Outer parallel region
#pragma omp parallel private(nthreads)
  {
  
    int idx = 0;

    nthreads = get_num_threads();    
    idx      = get_thread_id();
#pragma omp single
    printf("Outer parallel: %d\n\n",nthreads);
    
    // Inner parallel region
#pragma omp parallel num_threads(idx+1) private(nthreads)
    {
      nthreads = get_num_threads();
#pragma omp single
      printf("Inner parallel: %d\n",nthreads);
    }
#pragma omp barrier
#pragma omp single
    printf("\n");
  }


  return 0;
}

int get_num_threads() {
  
  static int accumulation  = 0;
  static int spin          = 0;

  int nthreads = -1;

  // Only masters enter, and keep spinning until other regions are finished
#pragma omp master
  {
    #pragma omp critical 
    {
      while(spin) {}
#pragma omp atomic
      spin++;
    }
  }

  // Compute the correct number of threads if 
  // called by all the threads in a parallel region at the same time
#pragma omp barrier
#pragma omp atomic
  accumulation++;
#pragma omp barrier
  nthreads = accumulation;
#pragma omp barrier
  accumulation = 0;
#pragma omp barrier
  
  // Master release lock
  #pragma omp master
  {
#pragma omp atomic
    spin--;
  }

  return nthreads;
}

int get_thread_id() {
  
  int nthreads = get_num_threads();
  int idx;

#pragma omp for schedule(static)
  for(int ii = 0; ii < nthreads; ii++)
    idx = ii;

  return idx;
}
