/* Exercise: Pi
 *
 * In this exercise you will determine the value 
 * of PI using the integral  of 
 *    4/(1+x*x) between 0 and 1. 
 *
 * The integral is approximated by a sum of n intervals.
 * 
 * The approximation to the integral in each interval is:
 *    (1/n)*4/(1+x*x). 
 */
#include<stdio.h>
#include<omp.h>
               
#define PI25DT 3.141592653589793238462643

#define INTERVALS 400000000

int main(int argc, char **argv) {

  size_t intervals = INTERVALS;

  const double dx = 1.0 / (double) intervals;
  double x, f, sum;
  double time[2] = {0.0f, 0.0f};
  
  time[0] = omp_get_wtime();
  
  printf("Number of intervals: %ld\n", intervals);

  sum = 0.0;  

#pragma omp parallel for schedule(static) reduction(+:sum) private(x,f)
  for (int i = 1; i <= intervals; i++) {
    x   = dx * ((double) (i - 0.5));
    f   = 4.0 / (1.0 + x*x);
    sum = sum + f;
  }
  
  double pi = dx*sum;
  
  time[1] = omp_get_wtime();
  
  printf("Computed PI %.24f\n", pi);
  printf("The true PI %.24f\n\n", PI25DT);
  printf("Elapsed time (s) = %.2lf\n", time[1] - time[0]);
  
  return 0;
}

// https://hpc-forge.cineca.it/files/ScuolaCalcoloParallelo_WebDAV/public/anno-2013/Summer-School/
